#include "player_actor.h"
#include "authority/game_state.h"
#include "authority/player_input.h"
#include "scene/main/viewport.h"

void PlayerActor::_bind_methods() {
}

void PlayerActor::ready() {
	PlayerInput *input{ cast_to<PlayerInput>(get_node(NodePath("%PlayerInput"))) };
	input->connect(PlayerInput::signal_movement, callable_mp(this, &self_type::input_move));
	set_process(true);
}

void PlayerActor::process(double delta) {
	Basis const basis{ this->get_viewport()->get_camera_3d()->get_global_basis() };
	Vector3 x{ basis.get_column(0) };
	x = Vector3{ x.x, 0.f, x.z }.normalized();
	Vector3 z{ basis.get_column(2) };
	z = Vector3{ z.x, 0.f, z.z }.normalized();
	set_movement_direction(x * this->move_input.x + z * this->move_input.y);
}

void PlayerActor::input_move(Vector2 movement) {
	this->move_input = movement;
}

void PlayerActor::_notification(int what) {
	if (Engine::get_singleton()->is_editor_hint()) {
		return;
	}
	switch (what) {
		default:
			return;
		case NOTIFICATION_READY:
			ready();
			return;
		case NOTIFICATION_PROCESS:
			process(get_process_delta_time());
			return;
	}
}
