#include "third_person_camera.h"
#include "player_input.h"

void ThirdPersonCamera::_bind_methods() {
}

void ThirdPersonCamera::ready() {
	set_process(true);
	if (PlayerInput * player_input{ cast_to<PlayerInput>(get_node(NodePath("%PlayerInput"))) }) {
		player_input->connect(PlayerInput::signal_look, callable_mp(this, &self_type::on_look_input));
		player_input->connect(PlayerInput::signal_mouselook, callable_mp(this, &self_type::on_mouselook_input));
	}
}

void ThirdPersonCamera::process(double delta) {
	this->look_rotation += look_rotation_motion * delta;
	this->look_rotation.y = CLAMP(this->look_rotation.y, -Math_PI / 2.5, Math_PI / 2.5);
	Vector3 pivot{ get_parent_node_3d()->get_global_position() };
	pivot.y += this->height;
	Vector3 offset{ 0.f, 0.f, this->distance };
	offset.rotate({ 1.f, 0.f, 0.f }, this->look_rotation.y);
	offset.rotate({ 0.f, 1.f, 0.f }, this->look_rotation.x);
	look_at_from_position(pivot + offset, pivot);
}

void ThirdPersonCamera::on_look_input(Vector2 input) {
	this->look_rotation_motion = input;
}

void ThirdPersonCamera::on_mouselook_input(Vector2 input) {
	this->look_rotation -= input;
}

void ThirdPersonCamera::_notification(int what) {
	if (Engine::get_singleton()->is_editor_hint()) {
		return;
	}
	switch (what) {
		case NOTIFICATION_READY:
			ready();
			break;
		case NOTIFICATION_PROCESS:
			process(get_process_delta_time());
			break;
	}
}

void ThirdPersonCamera::set_base_rotation_speed(Vector2 value) {
	this->base_rotation_speed = value;
}

Vector2 ThirdPersonCamera::get_base_rotation_speed() const {
	return this->base_rotation_speed;
}
